from __future__ import generators
import es
import playerlib
import gamethread

info = es.AddonInfo()
info.name        = "Sound Manager"
info.version     = "v1.0.1"
info.author      = "The Bash"
info.url         = "http://addons.eventscripts.com/addons/view/soundmanager"
info.basename    = "soundmanager"
info.description = "Provides greater assistance in managing sounds."

_globalVolume = 1.0
_globalAttenuation = 0.5

_sounds = {}
_soundGenerator = None

class Sound:
    __soundname = ""
    __filename = ""
    __volume = 1.0
    __attenuation = 0.5
    __length = None
    __playing = False
    __looping = False
    
    def __init__(self, newSoundname, newFilename, newVolume = 1.0, newAttenuation = 0.5, newLength = None):
        self.__soundname = newSoundname
        self.__filename = newFilename
        self.setVolume(newVolume)
        self.setAttenuation(newAttenuation)
        self.__length = newLength
        self.registerSound()
    
    def __del__(self):
        if self.__playing:
            self.stopSoundToAll()
            gamethread.cancelDelayed("SoundManager_ToPlayer_%s" % (self.__filename))
            gamethread.cancelDelayed("SoundManager_ToList_%s" % (self.__filename))
            gamethread.cancelDelayed("SoundManager_ToAll_%s" % (self.__filename))
    
    def registerSound(self):
        es.stringtable("downloadables", "sound/%s" % (self.__filename))
    
    def __triggerEvent(self, eventName, filterType, filterString):
        es.event("initialize", eventName)
        es.event("setstring", eventName, "soundname", self.__soundname)
        es.event("setstring", eventName, "filename", self.__filename)
        es.event("setfloat", eventName, "volume", self.__volume)
        es.event("setfloat", eventName, "attenuation", self.__attenuation)
        es.event("setfloat", eventName, "length", self.__length)
        es.event("setstring", eventName, "filtertype", filterType)
        es.event("setstring", eventName, "filterstring", filterString)
        es.event("fire", eventName)
    
    def getSoundname(self):
        return self.__soundname
        
    def getFilename(self):
        return self.__filename
    
    def getVolume(self):
        return self.__volume
    
    def setVolume(self, newVolume):
        if newVolume <= _globalVolume:
            self.__volume = newVolume
        else:
            self.__volume = _globalVolume
    
    def getAttenuation(self):
        return self.__attenuation
    
    def setAttenuation(self, newAttenuation):
        if newAttenuation >= _globalAttenuation:
            self.__attenuation = newAttenuation
        else:
            self.__attenuation = _globalAttenuation
    
    def getLength(self):
        return self.__length
    
    def getPlaying(self):
        return self.__playing
    
    def getLooping(self):
        return self.__looping
    
    def setLooping(self, newLoop):
        if self.__length != None:
            self._looping = newLoop
            
    def playSoundToPlayer(self, playerID):
        es.playsound(playerID, self.__filename, self.__volume)
        self.__triggerEvent("sound_play", "player", str(playerID))   
        if self.__length != None:
            self.__playing = True
            gamethread.delayedname(self.__length, "SoundManager_ToPlayer_%s" % (self.__filename), self.stopSoundToPlayer, playerID)     
    
    def playSoundToList(self, listFilter):    
        for ply in playerlib.getPlayerList(listFilter):        
            es.playsound(ply.userid, self.__filename, self.__volume)
        self.__triggerEvent("sound_play", "list", listFilter)
        if self.__length != None:
            self.__playing = True
            gamethread.delayedname(self.__length, "SoundManager_ToList_%s" % (self.__filename), self.stopSoundToList, listFilter)     
    
    def playSoundToAll(self):     
        for ply in es.getUseridList():
            es.playsound(ply, self.__filename, self.__volume)
        self.__triggerEvent("sound_play", "all", "")
        if self.__length != None:
            self.__playing = True
            gamethread.delayedname(self.__length, "SoundManager_ToAll_%s" % (self.__filename), self.stopSoundToAll)     

    def emitSoundFromPlayer(self, playerID):
        es.emitsound("player", playerID, self.__filename, self.__volume, self.__attenuation)
        self.__triggerEvent("sound_emit", "player", str(playerID))
    
    def emitSoundFromEntity(self, entityIndex):
        es.emitsound("entity", entityIndex, self.__filename, self.__volume, self.__attenuation)
        self.__triggerEvent("sound_emit", "entity", str(entityIndex))

    def stopSoundToPlayer(self, playerID):
        es.stopsound(playerID, self.__filename)
        self.__triggerEvent("sound_stop", "player", str(playerID))
        if self.__playing and self.__looping:
            self.playSoundToPlayer(playerID)
        else:
            self.__playing = False
            
    def stopSoundToList(self, listFilter):
        for ply in playerlib.getPlayerList(listFilter):        
            es.stopsound(ply.userid, self.__filename)
        self.__triggerEvent("sound_stop", "list", listFilter)
        if self.__playing and self.__looping:
            self.playSoundToList(listFilter)
        else:
            self.__playing = False
        
    def stopSoundToAll(self):
        for ply in es.getUseridList():
            es.stopsound(ply, self.__filename)
        self.__triggerEvent("stop_stop", "all", "")
        if self.__playing and self.__looping:
            self.playSoundToAll()
        else:
            self.__playing = False

def load():
    es.set("soundmanager_version",info.version)
    es.makepublic("soundmanager_version")
    
    es.loadevents("declare", "addons/eventscripts/soundmanager/soundmanagerevents.res")
    
    if not es.exists("command", "sm_add_sound"):
        es.regcmd("sm_add_sound", "soundmanager/addSoundFromConsole")
    if not es.exists("command", "sm_remove_sound"):
        es.regcmd("sm_remove_sound", "soundmanager/removeSoundFromConsole")
    if not es.exists("command", "sm_global_volume"):
        es.regcmd("sm_global_volume", "soundmanager/setGlobalVolumeFromConsole")
    if not es.exists("command", "sm_global_attenuation"):
        es.regcmd("sm_global_attenuation", "soundmanager/setGlobalAttenuationFromConsole")
    if not es.exists("command", "sm_play_sound"):
        es.regcmd("sm_play_sound", "soundmanager/playSoundFromConsole")
    if not es.exists("command", "sm_emit_sound"):
        es.regcmd("sm_emit_sound", "soundmanager/emitSoundFromConsole")
    if not es.exists("command", "sm_stop_sound"):
        es.regcmd("sm_stop_sound", "soundmanager/stopSoundFromConsole")
    
    _sounds.clear()
    es.dbgmsg(0, "[SoundManager]: Loaded")

def unload():
    _sounds.clear()
    es.dbgmsg(0, "[SoundManager]: Unloading")

def es_map_start(event_var):
    es.loadevents("addons/eventscripts/soundmanager/soundmanagerevents.res")
    for sound in _sounds.values():
        sound.registerSound()

def addSound(newSoundname, newFilename, newVolume = 1.0, newAttenuation = 0.5, newLength = None):
    if newSoundname in _sounds.keys():
        es.dbgmsg(0, "[SoundManager]: Cannot add new sound with name %s - this name has already been used." % (newSoundname))
    else:
        newSound = Sound(newSoundname, newFilename, newVolume, newAttenuation, newLength)
        _sounds[newSoundname] = newSound
        es.dbgmsg(0, "[SoundManager]: Added new sound with name %s." % (newSoundname))
        return newSound

def addSoundFromConsole():
    newSoundname = None
    if es.getargc() > 1:
        newSoundname = es.getargv(1)
    else:
        es.dbgmsg(0, "[SoundManager]: Could not add sound by 'add_sound': No sound name parameter found.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'add_sound <soundname> <filename> [volume] [attenuation] [length]'")
        return
        
    if newSoundname in _sounds.keys():
        es.dbgmsg(0, "[SoundManager]: Cannot add new sound with name %s - this name has already been used." % (newSoundname))
        return
    
    newFilename = None
    if es.getargc() > 2:
        newFilename = es.getargv(2)
    else:
        es.dbgmsg(0, "[SoundManager]: Could not add sound by 'add_sound': No file name parameter found.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'add_sound <soundname> <filename> [volume] [attenuation] [length]'")
        return
    
    newSound = Sound(newSoundname, newFilename)
    
    if es.getargc() > 3:
        newSound.setVolume(float(es.getargv(3)))
        if es.getargc() > 4:
            newSound.setAttenuation(float(es.getargv(4)))
            if es.getargc() > 5:
                newSound.setLength(float(es.getargv(5)))
    
    _sounds[newSoundname] = newSound
    es.dbgmsg(0, "[SoundManager]: Added new sound with name %s." % (newSoundname))

def removeSound(sound):
    if sound in _sounds.keys():
        del _sounds[sound]
    elif sound in _sounds.values():
        toBeDeleted = None
        for soundName in _sounds:
            if sound == _sounds[soundName]:
                toBeDeleted =  _sounds[soundName]
                break
        
        if toBeDeleted != None:
            del toBeDeleted
    else:
        es.dbgmsg(0, "[SoundManager]: Could not find sound to be removed.")

def removeSoundFromConsole():
    if es.getargc() > 1:
        removeSound(es.getargv(1))
    else:
        es.dbgmsg(0, "[SoundManager]: Could not remove sound by 'remove_sound': No sound name parameter found.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'remove_sound <soundname>'")

def getSound(soundname):
    if soundname in _sounds.keys():
        return _sounds[soundname]
    else:
        es.dbgmsg(0, "[SoundManager]: Could not retrieve sound.")
        return None

def getGlobalVolume():
    return _globalVolume

def setGlobalVolume(newVolume):
    global _globalVolume
    _globalVolume = newVolume
    for sound in _sounds.values():
        if sound.getVolume() > _globalVolume:
            sound.setVolume(_globalVolume)

def setGlobalVolumeFromConsole():
    if es.getargc() > 1:
        setGlobalVolume(float(es.getargv(1)))
    else:
        es.dbgmsg(0, "[SoundManager]: Could not set global volume by 'global_volume': No volume parameter found.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'global_volume <volume>'")

def getGlobalAttenuation():
    return _globalAttenuation
    
def setGlobalAttenuation(newAttenuation):
    global _globalAttenuation
    _globalAttenuation = newAttenuation
    for sound in _sounds.values():
        if sound.getAttenuation() > _globalAttenuation:
            sound.setAttenuation(_globalAttenuation)

def setGlobalAttenuationFromConsole():
    if es.getargc() > 1:
        setGlobalAttenuation(float(es.getargv(1)))
    else:
        es.dbgmsg(0, "[SoundManager]: Could not set global attenuation by 'global_attenuation': No attenuation parameter found.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'global_attenuation <attenuation>'")

def _soundIterator():
    for sound in _sounds:
        yield _sounds[sound]
        
def getSoundnameList():
    soundList = []
    for sound in _sounds:
        soundList.append(sound)
    
    return soundList
        
def getNextSound():
    global _soundGenerator
    if _soundGenerator == None:
        _soundGenerator = _soundIterator()
    
    nextSound = None
    try:
        nextSound = _soundGenerator.next()
    except:
        _soundGenerator = _soundIterator()
        try:
            nextSound = _soundGenerator.next()
        except:
            nextSound = None
    
    return nextSound

def playSoundFromConsole():
    if es.getargc() > 2:
        sound = getSound(es.getargv(1))
        if es.getargv(2) == "all":
            if sound != None:
                sound.playSoundToAll()
    elif es.getargc() > 3:
        sound = getSound(es.getargv(1))
        if es.getargv(2) == "player":
            if sound != None:
                sound.playSoundToPlayer(es.getargv(3))
        elif es.getargv(2) == "list":
            if sound != None:
                sound.playSoundToList(es.getargv(3))
        else:
            es.dbgmsg(0, "[SoundManager]: Could not play sound by 'play_sound': Arguments are not correct.")
            es.dbgmsg(0, "[SoundManager]: Argument 2 should be one of: 'player', 'list', or 'all'.")
    else:
        es.dbgmsg(0, "[SoundManager]: Could not play sound by 'play_sound': Arguments are not correct.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'play_sound <soundname> <player|list|all> [filter]")

def emitSoundFromConsole():
    if es.getargc() > 3:
        sound = getSound(es.getargv(1))
        if es.getargv(2) == "player":
            if sound != None:
                sound.emitSoundFromPlayer(es.getargv(3))
        elif es.getargv(2) == "entity":
            if sound != None:
                sound.emitSoundFromEntity(es.getargv(3))
        else:
            es.dbgmsg(0, "[SoundManager]: Could not emit sound by 'emit_sound': Arguments are not correct.")
            es.dbgmsg(0, "[SoundManager]: Argument 2 should be one of: 'player', or 'entity'.")
    else:
        es.dbgmsg(0, "[SoundManager]: Could not emit sound by 'emit_sound': Arguments are not correct.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'emit_sound <soundname> <player|entity> <filter>")

def stopSoundFromConsole():
    if es.getargc() > 2:
        sound = getSound(es.getargv(1))
        if es.getargv(2) == "all":
            if sound != None:
                sound.stopSoundToAll()
    elif es.getargc() > 3:
        sound = getSound(es.getargv(1))
        if es.getargv(2) == "player":
            if sound != None:
                sound.stopSoundToPlayer(es.getargv(3))
        elif es.getargv(2) == "list":
            if sound != None:
                sound.stopSoundToList(es.getargv(3))
        else:
            es.dbgmsg(0, "[SoundManager]: Could not stop sound by 'stop_sound': Arguments are not correct.")
            es.dbgmsg(0, "[SoundManager]: Argument 2 should be one of: 'player', 'list', or 'all'.")
    else:
        es.dbgmsg(0, "[SoundManager]: Could not stop sound by 'stop_sound': Arguments are not correct.")
        es.dbgmsg(0, "[SoundManager]: Should be of the form: 'stop_sound <soundname> <player|list|all> [filter]")
